function [ lineDiam_lastParams, locLineDist, histCounts, histEdges, CUT_LOC_BOOL_VECT ] = fitCurveOnSurface(LOC_X, LOC_Y, PSFSize, initParams, fitToMinBool, doubleLinesBool, linFitMethod, automaticProcessBool)
% This function fit a curve on a map created by Gaussian blurring the
% localizations (a "surface" in a 3D diagramm). In case of single line fit
% the curve fitting is be performed on the ridge of the line while in
% case of a double line it is performed on the gap.
% At first, it fits a straight line on the localizations with the
% "initLineFitting" function.
% After that, a figure is created depicting the fitted straight line, the
% region to be cut out and the localizations with the "manualLinearSetting"
% function.
% In the next step, the localizations are being cut out in the set region,
% then a map is created by applying Gaussian blur on these localizations.
% Then, the central curve fitting is executed on this map, with the given
% polynomial degree. A 3D diagramm depicting the fit is also creating in
% top view.
% Finally the 'central curve-localization' distance is calculated and the
% data is ejected and a histogram showing it is created.
% 
% parents: rainSTORM_extras_lineDiameter
% children: segLineDiam
% 
% INPUT VARIABLES
% LOC_X: "x" coordinates of the localizations
% LOC_Y: "y" coordinates of the localizations
% initParams: initial parameters for the straight line fitting and for the region to be cut out
% twoLinesBool: boolean for whether the curve fitting should be executed on the gap of a double line ("true") or on the ridge of a single line ("false")
% polyDegree: polynomial degree of the curve to be fit
% 
% OUTPUT VARIABLES
% lineDiam_lastParams: parameters of the straight line fitting and for the region to be cut out, initial parameters for the next function call
% locLineDist: 'central curve-localization' distances
% histCounts: histogram counts of the 'central curve-localization' distances
% histEdges: edges of the histogram of the 'central curve-localization' distances
% CUT_LOC_BOOL_VECT: boolean vector about which localizations fell within the cut-out region around the line


    function valuesSum=maxSum(xCoords, yCoords, meshData, sign, coeffs, init_coeffs)
        % function to fit the curve along the surface ninima or maxima
        % xCoords: "x" component of mesh coordinates
        % yCoords: "y" component of mesh coordinates
        % coeffs: coeffcients of the fitted line or curve
        
        stepSize=abs(coeffs-coeff_scaling_centralLine);
        for idxCoeffs=1:numel(coeffs)
            if stepSize(idxCoeffs)~=0 && coeff_scaling_centralLine(idxCoeffs)==0
                coeff_scaling_centralLine(idxCoeffs)=0.01*(1e-3^(numel(coeffs)-idxCoeffs))/stepSize(idxCoeffs);
            end
        end
        
        coeffs_scaled=init_coeffs+(coeffs-init_coeffs).*coeff_scaling_centralLine;
        
        yCoordsInterp=polyval(coeffs_scaled, xCoords);  % "y" sampled coordinates of the line or curve (let the "x" coordinates be the mesh "x" coordinates)
        lineValues=interp2(xCoords, yCoords, meshData, xCoords, yCoordsInterp);    % intepolating along the line or curve
        lineValues(isnan(lineValues))=0.5*(1+sign)*max(max(meshData));    % if it could not interpolate, i.e. the line or curve is out of the mesh, let it be
        valuesSum=sign*sum(lineValues)*1000;         % summing the interpolated values and negating them for the "fminsearch", let their absolute square value be summed in order to make the fitting more precise
    end

%% Initialization of some variables
xrange=initParams.xrange;
yrange=initParams.yrange;
sig=initParams.sig;
surfFittingPower=initParams.surfFittingPower;

%% Custom linear fitting of the initial line
[initLine_linearFit]=initLineFitting(LOC_X, LOC_Y, xrange, yrange, sig, surfFittingPower, linFitMethod, doubleLinesBool, fitToMinBool);

%% Setting initial line manually and adjusting the region to be cut out
[LOC_X_filtered, LOC_Y_filtered, lineDiam_lastParams, x1, y1, initLineFigure]=manualLinearSetting(LOC_X, LOC_Y, PSFSize, initLine_linearFit, initParams, automaticProcessBool);

% passing the manuially set parameters
alpha_mean=lineDiam_lastParams.alpha_mean;
xrange=lineDiam_lastParams.xrange;
yrange=lineDiam_lastParams.yrange;
sig=lineDiam_lastParams.sig;
sampLineDistThreshold=lineDiam_lastParams.sampLineDistThreshold;
nBins=lineDiam_lastParams.nBins;
polyDegree=initParams.polyDegree;
surfFittingPower=initParams.surfFittingPower;

xCenter=x1(2);    % "x" components of the data points' mean position
yCenter=y1(2);    % "y" components of the data points' mean position

%%  Cutting out the localizations
% Rotation of coordinate system with alpha, and calculation of new X
% coordinates
% LOC_X_ALPHA=(LOC_X-xCenter).*cos(alpha_mean+90*pi/180)+(LOC_Y-yCenter).*sin(alpha_mean+90*pi/180);
% LOC_Y_ALPHA=(LOC_X-xCenter).*sin(alpha_mean+90*pi/180)-(LOC_Y-yCenter).*cos(alpha_mean+90*pi/180);
LOC_X_ALPHA=(LOC_X_filtered-xCenter).*cos(alpha_mean)+(LOC_Y_filtered-yCenter).*sin(alpha_mean);
LOC_Y_ALPHA=-(LOC_X_filtered-xCenter).*sin(alpha_mean)+(LOC_Y_filtered-yCenter).*cos(alpha_mean);
%LOC_X_ALPHA=LOC_X;
%LOC_Y_ALPHA=LOC_Y;

% Store the rotated coordinates:
lineDiam_lastParams.LOC_X_ALPHA = LOC_X_ALPHA;
lineDiam_lastParams.LOC_Y_ALPHA = LOC_Y_ALPHA;

% Cutting the out the localizations in the selected region
CUT_LOC_BOOL_VECT=LOC_X_ALPHA>=-xrange & LOC_X_ALPHA<=xrange & LOC_Y_ALPHA>=-yrange & LOC_Y_ALPHA<=yrange;
LOC_X_ALPHA_CUT=LOC_X_ALPHA(CUT_LOC_BOOL_VECT);
LOC_Y_ALPHA_CUT=LOC_Y_ALPHA(CUT_LOC_BOOL_VECT);

%% Creating the map, Gaussian blurring the localizations
% step size of the resampling mesh
deltaX=5;
deltaY=5;

% number of resampling "x" and "y" coordinates on the selected region
numX=ceil(2*xrange/deltaX);
numY=ceil(2*yrange/deltaY);

% resampling "x" and "y" coordinates on the selected region
X_ALPHA=((-(numX-1)*deltaX*0.5):deltaX:((numX-1)*deltaX*0.5));
Y_ALPHA=((-(numY-1)*deltaY*0.5):deltaY:((numY-1)*deltaY*0.5))';

% matrices consisting the coordinates of the resampling mesh
mat_X=repmat(X_ALPHA,size(Y_ALPHA));
mat_Y=repmat(Y_ALPHA,size(X_ALPHA));

% applying Gaussian blur on the localizations
F=zeros(size(mat_X));
sigma=sig;
for idx=1:length(LOC_X_ALPHA_CUT)
    F=F+exp((-(mat_X-LOC_X_ALPHA_CUT(idx)).^2-(mat_Y-LOC_Y_ALPHA_CUT(idx)).^2)/(2*sigma^2));
end
F=F.^surfFittingPower;

% plotting the mesh
%figure(11)
fittedCurveFigure=figure('visible','off');
set(gcf, 'Name', 'Fit curve along the highest localization density')
mesh(X_ALPHA,Y_ALPHA,F)
hold on;

% fitting the surface with "fminsearch", "fmincon" could be another solution
%init_coeffs=[tan(alpha_mean-90*pi/180), centerY-centerX*tan(alpha_mean-90*pi/180)];
coeffs_N=polyDegree+1;     % number of the coefficents of the polynomial fit, degree of the polynomial to be fit plus 1

%% Fitting the curve
% default initial coefficients
init_coeffs=zeros(1,coeffs_N);

%options = optimset('Display','off');        % do not display iteration output
% the defalut step size for the iteration is 0.25*1e-3 (only in case of [0,0,0] initial parameters?), the zeroth order
% term requires 3 orders of magnitude greater, the second order term
% requires 3 orders of magnitude smaller
global coeff_scaling_centralLine;
coeff_scaling_centralLine=zeros(1,numel(init_coeffs));
options=optimset('FinDiffRelStep', [0.25*1e-6, 0.25*1e-3, 0.25*1e0]);       % WHY THE FUCK CAN'T MATLAB RESPECT THE MANUALLY SET STEP SIZES??????????, this does not work to be clear (it might pertain to the numerical derivation which the "fminsearch" does not seem to use, does it mean Matlab decides the step sizes arbitrarily and can not be adjusted?)

% sign for the fitting curve on the surface
switch fitToMinBool
    case false  % in case of a single line
        fitSign=-1;
    otherwise   % in case of a double line
        fitSign=1;
end

% fitting the curve along the surface minima or maxima
[fit_coeffs_temp,~,exitflag_fit]=fminsearch(@(coeffs) maxSum(X_ALPHA, Y_ALPHA, F, fitSign, coeffs, init_coeffs), init_coeffs, options);
fit_coeffs=init_coeffs+(fit_coeffs_temp-init_coeffs).*coeff_scaling_centralLine;    % had to do custom scaling the step size beacuse Matlab sucks
coeff_scaling_centralLine=zeros(1,numel(init_coeffs));
disp('Fitting curve is finished.')

X_ALPHA_CURVE=X_ALPHA;
Y_ALPHA_CURVE=polyval(fit_coeffs, X_ALPHA_CURVE);

% Store fit coefficients.
lineDiam_lastParams.fit_coeffs = fit_coeffs;
lineDiam_lastParams.X_ALPHA_CURVE = X_ALPHA_CURVE;
lineDiam_lastParams.Y_ALPHA_CURVE = Y_ALPHA_CURVE;

% plotting the curve with the initial coefficients
plot3(X_ALPHA_CURVE, polyval(init_coeffs, X_ALPHA_CURVE), max(max(F))*ones(size(X_ALPHA_CURVE)), 'green')

% plotting the fitted curve
plot3(X_ALPHA_CURVE, Y_ALPHA_CURVE, max(max(F))*ones(size(X_ALPHA_CURVE)), 'red')
view([0,0,90])
hold off
%% Determining the distance
% distance of localizations fro,m the central curve
locLineDist=segLineDiam(X_ALPHA_CURVE, Y_ALPHA_CURVE, LOC_X_ALPHA_CUT, LOC_Y_ALPHA_CUT, sampLineDistThreshold);

% histogram parameters
binLowerBound=-sampLineDistThreshold;   % lower bound of the histogram
binUpperBound=sampLineDistThreshold;    % upper bound of the histogram
binEdges=linspace(binLowerBound, binUpperBound, nBins);  % edges of the bins

% plot the histograms
distHistFigure=figure('visible','off');
set(gcf, 'Name', 'Histogram of distances.')
[histCounts, histEdges]=histcounts(locLineDist, binEdges);
histogram('BinEdges',histEdges,'BinCounts',histCounts)

% pass the figure handles
lineDiam_lastParams.fig10 = initLineFigure;
lineDiam_lastParams.fig11 = fittedCurveFigure;
lineDiam_lastParams.fig12 = distHistFigure;
end