function [initLine_linearFit] = initLineFitting(LOC_X, LOC_Y, xrange, yrange, sig, surfFittingPower, linFitMethod, doubleLinesBool, fitToMinBool)
% This function fit an initial straight line in several steps. The first
% step is a simple linear fitting either on the locaization (single or
% double lines) or on points sampled from the surface with value under a
% certain threshold. The second step is a more sophisticated (custom)
% linear fitting which requires some kind os numerical treatment. The third
% step is linear fitting either along the surface maxima (single line or 
% not that smeared double lines) or along the surface minima (gap or 
% smeared double lines)


    function valuesSum=maxSum_fitLin(xCoords, yCoords, meshData, sign, coeffs, init_coeffs_fitLin)
        % function to fit the linear along the surface ninima or maxima
        % xCoords: "x" component of mesh coordinates
        % yCoords: "y" component of mesh coordinates
        % coeffs: coeffcients of the fitted line or curve
        
        stepSize=abs(coeffs-init_coeffs_fitLin);
        for idxCoeffs=1:numel(coeffs)
            if stepSize(idxCoeffs)~=0 && coeff_scaling_initLine(idxCoeffs)==0
                coeff_scaling_initLine(idxCoeffs)=1.25*(1e-3^(numel(coeffs)-idxCoeffs))/stepSize(idxCoeffs);
            end
        end
        
        coeffs_scaled=init_coeffs_fitLin+(coeffs-init_coeffs_fitLin).*coeff_scaling_initLine;
        
        if abs(coeffs_scaled(1))<1
            xCoordsInterp=xCoords;
            yCoordsInterp=polyval(coeffs_scaled, xCoordsInterp);  % "y" sampled coordinates of the line or curve (let the "x" coordinates be the mesh "x" coordinates)
            
            ymin=min(yCoords);
            ymax=max(yCoords);
            withinBool=yCoordsInterp>=ymin & yCoordsInterp<=ymax;
            
        else
            coeffs_mod(1)=1/coeffs_scaled(1);
            coeffs_mod(2)=-coeffs_scaled(2)/coeffs_scaled(1);
            yCoordsInterp=yCoords;
            xCoordsInterp=polyval(coeffs_mod, yCoordsInterp);  % "y" sampled coordinates of the line or curve (let the "x" coordinates be the mesh "x" coordinates)
            
            xmin=min(xCoords);
            xmax=max(xCoords);
            withinBool=xCoordsInterp>=xmin & xCoordsInterp<=xmax;
        end
        lineValues=interp2(xCoords, yCoords, meshData, xCoordsInterp, yCoordsInterp);    % intepolating along the line or curve
        lineValues(isnan(lineValues))=0;    % if it could not interpolate, i.e. the line or curve is out of the mesh
        
        withinN=sum(withinBool);
        if withinN~=0
            %y_temp=yCoordsInterp(withinBool);
            %x_temp=xCoordsInterp(withinBool);
            %valuesSum=sign*sum(abs(lineValues).^2)*1000/sqrt((x_temp(end)-x_temp(1))^2+(y_temp(end)-y_temp(1))^2);         % summing the interpolated values and negating them for the "fminsearch", let their absolute square value be summed in order to make the fitting more precise
            %valuesSum=sign*sum(abs(lineValues).^2)*1000/(numel(x_temp));         % summing the interpolated values and negating them for the "fminsearch", let their absolute square value be summed in order to make the fitting more precise
            %valuesSum=sign*sum(lineValues)*1000/(numel(xCoordsInterp));
            valuesSum=sign*sum(lineValues)*1000/(withinN);
        else
            valuesSum=Inf;
        end
        
%         figure(20)
%         mesh(xCoords, yCoords, meshData)
%         hold on
%         % plotting the curve with the initial coefficients
%         plot3(xCoords, polyval(coeffs_scaled, xCoords), max(max(meshData))*ones(size(xCoords)), 'green')
%         view([0,0,90])
%         hold off
%         a=7;
    end


%% Preparing the localization data

%xrange_surf_initLine=xrange;
%yrange_surf_initLine=yrange;
%xrange_surf_initLine=1000;
%yrange_surf_initLine=1000;

% how the fitting should be performed in the last step when fitting on the
% localization density surface
switch linFitMethod
    case 'surfaceMin'
        % fit the linear on the gap or between the double lines
        xrange_surf_initLine=min(xrange, yrange);
        yrange_surf_initLine=min(xrange, yrange);
    case 'highestDensity'
        % fit linear along the highest localization densities on a single
        % line or on a smeared double line
        xrange_surf_initLine=max(xrange, yrange);
        yrange_surf_initLine=max(xrange, yrange);
end

% center coordinates of the structure
%xcentre_initLine=(min(LOC_X)+max(LOC_X))/2;
%ycentre_initLine=(min(LOC_Y)+max(LOC_Y))/2;
xcentre_initLine=mean(LOC_X);   % let the mean be the center
ycentre_initLine=mean(LOC_Y);   % let the mean be the center

%% Creating a surface out of the localizations

% step size of the resampling mesh
deltaX_initLine=10;
deltaY_initLine=10;

% number of resampling "x" and "y" coordinates on the selected region
numX_initLine=ceil(2*xrange_surf_initLine/deltaX_initLine);
numY_initLine=ceil(2*yrange_surf_initLine/deltaY_initLine);


% resampling "x" and "y" coordinates on the selected region
X_initLine=((-(numX_initLine-1)*deltaX_initLine*0.5):deltaX_initLine:((numX_initLine-1)*deltaX_initLine*0.5));
Y_initLine=((-(numY_initLine-1)*deltaY_initLine*0.5):deltaY_initLine:((numY_initLine-1)*deltaY_initLine*0.5))';

% matrices consisting the coordinates of the resampling mesh
mat_X_initLine=repmat(X_initLine,size(Y_initLine));
mat_Y_initLine=repmat(Y_initLine,size(X_initLine));

% applying Gaussian blur on the localizations to create a surface
Surf_linear=zeros(size(mat_X_initLine));
% standard deviation of the Gaussian blurr:
%sigma_initLine=(xrange_fitLin+yrange_fitLin)/16;
if doubleLinesBool && fitToMinBool
    % when examining the double lines and do not want to smear the two
    % lines when performing the general curve fitting (later function)
    switch linFitMethod
        case 'surfaceMin'
            sigma_initLine=sig*1.0;       % let it be, it should not smear the two lines
            fitSign=1;
        case 'highestDensity'
            sigma_initLine=sig*3.0;       % let it be, it should smear the two lines
            fitSign=-1;
    end
    
else
    sigma_initLine=sig*1.5;       % let it be, it was suitable for the "M" line of the myosin
    
    % sign for the fitting line on the surface
    switch fitToMinBool && ~doubleLinesBool
        case false  % in case of a single line or double line
            fitSign=-1;
        otherwise   % in case of a gap
            fitSign=1;
    end
    
end

% creating the surface:
for idx=1:length(LOC_X)
    Surf_linear=Surf_linear+exp((-(mat_X_initLine-(LOC_X(idx)-xcentre_initLine)).^2-(mat_Y_initLine-(LOC_Y(idx)-ycentre_initLine)).^2)/(2*sigma_initLine^2));
end
Surf_initLine=Surf_linear.^surfFittingPower;

% figure()
% surf(mat_X_initLine+xcentre_initLine, mat_Y_initLine+ycentre_initLine, Surf_initLine)

%% The first linear fitting with the built-in funtion
     
% defining a tigher region for the first fitting step
range_linearFit_initLine=min(xrange, yrange);
LOC_initLine_booleanVect=(LOC_X-xcentre_initLine).^2+(LOC_Y-ycentre_initLine).^2<range_linearFit_initLine^2;
LOC_X_initLine_centered=LOC_X(LOC_initLine_booleanVect)-xcentre_initLine;
LOC_Y_initLine_centered=LOC_Y(LOC_initLine_booleanVect)-ycentre_initLine;

if strcmp(linFitMethod, 'surfaceMin') && ~ doubleLinesBool
        % in case of gaps, sample points from the surface with value under
        % a certain threshold on which the line fitting can be performed
        Surf_outOfRange_boolMat=(mat_X_initLine.^2 +mat_Y_initLine.^2)/range_linearFit_initLine^2>1;
        
        % minimum and maximum values of the surface in the range
        Surf_linear_min=min(Surf_linear(~Surf_outOfRange_boolMat));
        Surf_linear_max=max(Surf_linear(~Surf_outOfRange_boolMat));
        
        % sampled the surface under a certain threshold
        Surf_initLine_gap_boolMat=Surf_linear<(Surf_linear_min+(Surf_linear_max-Surf_linear_min)*0.05);
        Surf_initLine_gap_boolMat_linIndices=find(Surf_initLine_gap_boolMat & ~Surf_outOfRange_boolMat);
        X_coords_gap=mat_X_initLine(Surf_initLine_gap_boolMat_linIndices);
        Y_coords_gap=mat_Y_initLine(Surf_initLine_gap_boolMat_linIndices);
        
        % the sampled points
        initLine_X_coords=X_coords_gap;
        initLine_Y_coords=Y_coords_gap;
else
    % in case of single or double lines, use the localization data
        initLine_X_coords=LOC_X_initLine_centered;
        initLine_Y_coords=LOC_Y_initLine_centered;
end

% CALCULATING THE FITTED LINE ANALYICALLY, f(x)=m*x+h
% fitting linear to the data point to estimate the lines' direction

% Matlab's bult-in function, requires the Curve Fitting Toolbox
% fo = fitoptions('Method','LinearLeastSquares');
% buitIn_linearFit=fit(initLine_X_coords,initLine_Y_coords,'poly1', fo);     % linear fit
% initLineFit.m=buitIn_linearFit.p1;
% initLineFit.h=buitIn_linearFit.p2;

% custon fitting
N=length(initLine_X_coords);
sum_X_i=sum(initLine_X_coords);
sum_Y_i=sum(initLine_Y_coords);
sum_X_i_2=sum(initLine_X_coords.^2);
sum_X_i_Y_i=sum(initLine_X_coords.*initLine_Y_coords);

m=(N*sum_X_i_Y_i-sum_Y_i*sum_X_i)/(N*sum_X_i_2-sum_X_i*sum_X_i);
h=(sum_Y_i-m*sum_X_i)/N;

initLineFit.m=m;
initLineFit.h=h;

%FOR TESTING:
% figure(7)
% ax=axes;
% scatter(initLine_X_coords,initLine_Y_coords, 5, '.', 'r')  % plotting the data points
% %view(90, 90)                    % changing the two axes
% xlim=get(ax, 'XLim');
% ylim=get(ax, 'YLim');
% set(gca, 'XLim', xlim, 'YLim', ylim);   % fixing the limit on the axes, needs bceause the line plot, the axis() function does not work correctly...
% daspect([1 1 1])                        % fixing the aspect ration, the axis('equal') and axis('image') does not work correctly
% hold on;
% linFunc_x=linspace(xlim(1), xlim(2), 100);
% linFunc_y=m*linFunc_x+h;
% plot(linFunc_x, linFunc_y,'b')             % plotting the fitted linear
% legend('off');

% the coefficients in a vector
coeffs_initLine_1=[initLineFit.m, initLineFit.h];

%% The second linear fitting with a custom function

[coeffs_initLine_2, exitflag]=linearFitting_fsolve(initLine_X_coords, initLine_Y_coords, coeffs_initLine_1);
% if the fitting was not successfull, fall back to the simpler method
if exitflag==-1
    coeffs_initLine_2=coeffs_initLine_1;
end

% if "gap" sample sample is used, then the originally fitted line might be needed to be rotated by 90° degrees
switch linFitMethod
    case {'surfaceMin'}
        % do nothing
    case 'highestDensity'
        % in case of gap fitting
        if fitToMinBool && ~doubleLinesBool
            % rotate the fitted line with 90° degrees, as it usually fits a
            % perpencicular one compard to the desired one
            coeffs_initLine_2(1)=tan(atan(coeffs_initLine_2(1))+90*pi/180);
        else
           % do nothing 
        end
end

%FOR TESTING:
% figure(8)
% ax=axes;
% scatter(initLine_X_coords,initLine_Y_coords, 5, '.', 'r')  % plotting the data points
% %view(90, 90)                    % changing the two axes
% xlim=get(ax, 'XLim');
% ylim=get(ax, 'YLim');
% set(gca, 'XLim', xlim, 'YLim', ylim);   % fixing the limit on the axes, needs bceause the line plot, the axis() function does not work correctly...
% daspect([1 1 1])                        % fixing the aspect ration, the axis('equal') and axis('image') does not work correctly
% hold on;
% initLine_linearFit.p1=coeffs_initLine_2(1);
% initLine_linearFit.p2=coeffs_initLine_2(2);
% linFunc_x=linspace(xlim(1), xlim(2), 100);
% linFunc_y=initLine_linearFit.p1*linFunc_x+initLine_linearFit.p2;
% plot(linFunc_x, linFunc_y,'b')             % plotting the fitted linear
% legend('off');
% hold off;

%% The third linear fitting on the created surface

%options = optimset('Display','off');        % do not display iteration output
% the defalut step size for the iteration is 0.25*1e-3, the zeroth order
% term requires 3 orders of magnitude greater, the second order term
% requires 3 orders of magnitude smaller
global coeff_scaling_initLine;
coeff_scaling_initLine=[1,1];
options=optimset('FinDiffRelStep', [0.25*1e-3, 0.25*1e0], 'Display','off');       % WHY THE FUCK CAN'T MATLAB RESPECT THE MANUALLY SET STEP SIZES??????????
fval_init=maxSum_fitLin(X_initLine, Y_initLine, Surf_initLine, fitSign, coeffs_initLine_2, coeffs_initLine_2); % initial value of the functional to be minimized
% straight fitting on the surface
coeff_scaling_initLine=[0,0];
[coeffs_initLine_3_temp, fval_fitLin, exitflag_fit_fitLin]=fminsearch(@(coeffs) maxSum_fitLin(X_initLine, Y_initLine, Surf_initLine, fitSign, coeffs, coeffs_initLine_2), coeffs_initLine_2, options);
coeffs_initLine_3=coeffs_initLine_2+(coeffs_initLine_3_temp-coeffs_initLine_2).*coeff_scaling_initLine;    % had to do custom scaling of the step size beacuse Matlab sucks
coeff_scaling_initLine=[0,0];

%FOR TESTING:
% figure(9)
% mesh(X_initLine,Y_initLine,Surf_initLine)
% hold on;
% Y_CURVE_fitLin=polyval(coeffs_initLine_2, X_initLine);
% plot3(X_initLine, Y_CURVE_fitLin, max(max(Surf_initLine))*ones(size(X_initLine)), 'red')
% hold on;
% Y_CURVE_fitLin=polyval(coeffs_initLine_3, X_initLine);
% plot3(X_initLine, Y_CURVE_fitLin, max(max(Surf_initLine))*ones(size(X_initLine)), 'blue')
% hold off;


% is the straight fitting on the surface was successful
if (exitflag_fit_fitLin==1 || exitflag_fit_fitLin==0) && fval_fitLin<=fval_init
%if false
    initLine_linearFit.m=coeffs_initLine_3(1);
    initLine_linearFit.h=coeffs_initLine_3(2)+ycentre_initLine-xcentre_initLine*coeffs_initLine_3(1);
% if not, use the origially fitted and then rotated straight line
else
    % let it be the conventional linear fitting
    initLine_linearFit.m=coeffs_initLine_2(1);
    initLine_linearFit.h=coeffs_initLine_2(2)+ycentre_initLine-xcentre_initLine*coeffs_initLine_2(1);
end

end

