function [locLineDist] = segLineDiam(X_CURVE, Y_CURVE, LOC_X, LOC_Y, sampLineDistThreshold)
% This function calculates the distance between the localized points of a
% sample (its coordinates are given by the "LOC_X" and the "LOC_Y" input
% variables) and a central segmented line (the coordinates of its vertices
% are described by the "X_CURVE", "Y_CURVE" input variables).
% First, calculates the unit vectors along each segment, the vectors (not
% unit vectors) are pointing away from the segmented line at each vertex,
% on the same side of the line. After that, it determines the distance
% between each localized sample point and the segmented line. First, it
% estimates which segment the actual sample point belong to. Then, finds
% the segment which the actual sample point belongs to by "going around"
% the initial segment (moving more and more away from it and altering
% sides). After it found the right segment, finds out whether its
% projection falls into the segment, if yes it calculates the sample point's
% distance from the line determined by the segment, if not it calculates
% the sample point's distance from the right vertex. All localization that
% is farther from the segmented line than the thresholding
% "sampLineDistThreshold" input variable are discarded

% parents: fitCurveOnSurface
% childs: -

% the segmented line
lineVertN=numel(X_CURVE(:));
lineVertCoords(3,:)=zeros(1,lineVertN);
lineVertCoords(1,:)=X_CURVE(:);
lineVertCoords(2,:)=Y_CURVE(:);

% the localizations
sampN=numel(LOC_X);
sampCoords(3,:)=zeros(1,sampN);
sampCoords(1,:)=LOC_X(:);
sampCoords(2,:)=LOC_Y(:);

% vectors along the segments of the segmented line
lineSegVect=lineVertCoords(:,2:end)-lineVertCoords(:,1:end-1);
lineSegVect=lineSegVect./repmat(sqrt(sum(lineSegVect.^2,1)),3,1);        % normalization

% matrix of 90� rotation in "x-y" plane, used for creating perpendicular vectors
rotMat=[...
    0,1,0;...
    -1,0,0;...
    0,0,1;...
    ];

% vectors belonging to the vertices, pointing away from the segmented line,
% on the same side of the segment
lineVertVect=zeros(3,lineVertN);
lineVertVect(:,2:end-1)=lineSegVect(:,1:end-1)-lineSegVect(:,2:end);    % let let be the subtraction of the unit vectors of two adjacent segment
lineVertVect(:,1)=rotMat*lineSegVect(:,1);      % for the first vertex let it be perpendicular to the first segment
lineVertVect(:,end)=rotMat*lineSegVect(:,end);  % for the last vertex let it be perpendicular to the last segment
% if the two adjacent segments were parallel (using <1e-10 intead of ==0
% because of numerical errors)m let it be perpendicular to them
lineVertVect(:,sum(lineVertVect.^2,1)<1e-10)=rotMat*lineSegVect(:,sum(lineVertVect.^2,1)<1e-10);

% vector conatining signs for the vectors of the vertices, all should point
% away from the same side
signVectVert=ones(3, lineVertN);
for idxVert=2:lineVertN-1   % loop for all vertice except for the first and the last
    temp=cross(lineSegVect(:,idxVert-1), lineVertVect(:,idxVert));  % cross product of the vectors of the segments and the vertices, its sign of the "z" coordinate determines whethet the vertex vector point to the right direction
    if temp(3)>0
        signVectVert(:, idxVert)=-1;    % minus sign for changing the vectors direction
    elseif temp(3)<0
        %signVectVert(:, idx)=1;    % no need to change the sign in this case
    else
        error('There is some problem with the vectors of the segment or of the vertices, they should not be parallel.')    % this condition should not be fullfilled
    end
end
% correcting the direction of the vectors of the vertices
lineVertVect=lineVertVect.*signVectVert;

% vector for storing the distance between the segmented line and the points
% of the sample
locLineDist=zeros(sampN,1);
for idxSamp=1:sampN     % loop over all sample points, determining their distance
    % estimating wich segment the actual sample point might belong to
    temp1=lineVertCoords(1,:)-sampCoords(1,idxSamp);    % subtracting the actual sample point coordinate from the coordinates of all the vertices
    temp2=temp1(1:end-1).*temp1(2:end); % multiplying the adjacent "x" elements of the vector of the subtraction
    temp3=find(temp2<=0);   % index/indices where the subtraction change sign
    if length(temp3)==1 % if one index was found
        idxSeg_0=temp3; % let it be the initial segment index while searching which segment the actual sample pont belongs to
    elseif length(temp3)==2	% if two indices were found
        if temp2(temp3(1))==0 && temp2(temp3(2))==0 % if they are zero, the sample point is on a line determined by a vertex vector
            idxSeg_0=temp3(1);  % let the first found index be the initial segment index while searching which segment the actual sample pont belongs to
        else     % this should be met
            error('There is some problem with the "x" coordinates of the segmented line.')
        end
    elseif isempty(temp3)   % if no index was found, the localized point is out of the line "along the long axis"
        locLineDist(idxSamp)=NaN;
        continue;   % step to the next loop
    elseif length(temp3)>2  % if more indices were found, this should not happen
        error('The "x" coordinates of the segmented line''s vertices are not in sequential order.')
    end
    
    % while loop for determining which segment the actual sample point
    % belong to and its actual distance to the segmented line
    % the loop examnies the segments starting from the initial one, getting
    % more and more away from it and exchanging the sides in every step
    idxSegChange=0;     % for getting away from the initial segment
    idxSegChangeSign=1; % for changing the sides in every step
    idxSeg=uint16(idxSeg_0+idxSegChangeSign*idxSegChange);  % index of the actual segment
    idxSegPrev=idxSeg;  % index of the previous segment, needed for giving "NaN" value to the distance if the sample point of out of the segmented line "along the long axis"
    while idxSeg>=1 || idxSeg<=(lineVertN-1) || idxSegPrev>=1 || idxSegPrev<=(lineVertN-1)
        if idxSeg>=1 && idxSeg<=(lineVertN-1)   % if the actual segment is on the segmented line, its index is valid
            % cross product for determining whether the actual sample point
            % is between the the two lines determined by the two vertex
            % vectors belonging to the two vertice of the actual segment
            crossProd1=cross((sampCoords(:,idxSamp)-lineVertCoords(:,idxSeg)),lineVertVect(:,idxSeg));
            crossProd2=cross((sampCoords(:,idxSamp)-lineVertCoords(:,idxSeg+1)),lineVertVect(:,idxSeg+1));
            if crossProd1(3)*crossProd2(3)<0    % if the sample point is between the the two lines determined by the two vertex vectors belonging to the two vertice of the actual segment
                alpha=atan2(lineSegVect(2,idxSeg), lineSegVect(1,idxSeg));  % angle of incline of the line determined by the actual segment
                m=tan(alpha);   % slope of the line determined by the actual segment
                h=lineVertCoords(2,idxSeg)-m*lineVertCoords(1,idxSeg);  % y-intercept of the line determined by the actual segment
                yDist=sampCoords(2,idxSamp)-(m*sampCoords(1,idxSamp)+h);    % distance between the actual sample point and the line determined by the segment along the "y" direction
                
                % determining whether the actual sample point is between
                % the two line that areperpendicular to the actual segment and
                % intercepting it at the two ends
                % taking the projections of vectors pointing to the sample and the vertice
                % points from the y-intercent of the line determined by the
                % actual segment
                sampProj=transpose(sampCoords(:,idxSamp)-[0;h;0])*lineSegVect(:,idxSeg);
                vertProj_1=transpose(lineVertCoords(:,idxSeg)-[0;h;0])*lineSegVect(:,idxSeg);
                vertProj_2=transpose(lineVertCoords(:,idxSeg+1)-[0;h;0])*lineSegVect(:,idxSeg);
                if sampProj>=vertProj_1 && sampProj<=vertProj_2     % if the actual sample point is between the two line that are perpendicular to the actual segment and intercepting it at the two ends
                    locLineDist(idxSamp)=yDist*cos(alpha);   % sample point's distance from the line determined by the actual segment
                elseif sampProj<vertProj_1 % if the actual sample point is not between the two line that are perpendicular to the actual segment and intercepting it at the two ends
                    locLineDist(idxSamp)=yDist/abs(yDist)*norm(sampCoords(:,idxSamp)-lineVertCoords(:,idxSeg));   % sample point's distance from the right vertex
                elseif sampProj>vertProj_2 % if the actual sample point is not between the two line that are perpendicular to the actual segment and intercepting it at the two ends
                    locLineDist(idxSamp)=yDist/abs(yDist)*norm(sampCoords(:,idxSamp)-lineVertCoords(:,idxSeg+1));   % sample point's distance from the right vertex
                else    % this condition should not be met
                    error('Something is wrong while finding which segment the actual sample point belongs to.');
                end
                
                if abs(locLineDist(idxSamp))>sampLineDistThreshold
                    locLineDist(idxSamp)=NaN;
                end
                
                break
            elseif crossProd1(3)==0    % if the actual sample point is on one of the lines determined by the two vertex vectors belonging to the two vertice of the actual segment
                alpha=atan2(lineSegVect(2,idxSeg), lineSegVect(1,idxSeg));  % angle of incline of the line determined by the actual segment
                m=tan(alpha);   % slope of the line determined by the actual segment
                h=lineVertCoords(2,idxSeg)-m*lineVertCoords(1,idxSeg);  % y-intercept of the line determined by the actual segment
                yDist=sampCoords(2,idxSamp)-(m*sampCoords(1,idxSamp)+h);    % distance between the actual sample point and the line determined by the segment along the "y" direction
                
                locLineDist(idxSamp)=yDist/abs(yDist)*norm(sampCoords(:,idxSamp)-lineVertCoords(:,idxSeg)); % sample point's distance from the right vertex
                break   % step to the sample point
            elseif crossProd2(3)==0    % if the actual sample point is on one of the lines determined by the two vertex vectors belonging to the two vertice of the actual segment
                alpha=atan2(lineSegVect(2,idxSeg), lineSegVect(1,idxSeg));  % angle of incline of the line determined by the actual segment
                m=tan(alpha);   % slope of the line determined by the actual segment
                h=lineVertCoords(2,idxSeg)-m*lineVertCoords(1,idxSeg);  % y-intercept of the line determined by the actual segment
                yDist=sampCoords(2,idxSamp)-(m*sampCoords(1,idxSamp)+h);    % distance between the actual sample point and the line determined by the segment along the "y" direction
                
                locLineDist(idxSamp)=yDist/abs(yDist)*norm(sampCoords(:,idxSamp)-lineVertCoords(:,idxSeg+1));   % sample point's distance from the right vertex
                break;  % step to the sample point
            else
                error('');
            end
        elseif idxSegPrev<1 && idxSeg>(lineVertN-1) % if the sample point of out of the segmented line "along the long axis", checked at both ends
            locLineDist(idxSamp)=NaN;              % it should be "NaN"
            break;                                  % step to the sample point
        end
        
        idxSegChangeSign=-idxSegChangeSign;         % change side in searching the right segment in the next loop
        if idxSegChangeSign==1              % if the sign is "1" again
            idxSegChange=idxSegChange+1;    % get more away from the initial segment, too
        end
        idxSegPrev=idxSeg;      % the index of the previous segment in the next loop
        idxSeg=uint16(idxSeg_0+idxSegChangeSign*idxSegChange);  % index of segment in the next loop
    end
    
end

end