% 	Summary:	Line separator tool for SzikoraSz.
%	Version:	0.3
%	Note:		
%	Package:	rainSTORM v3.1+ (2017-07)    
%	Developer:	adoptim
%	Name:		SZTE Advanced Optical Imaging Group
%	Company:	University of Szeged

function rainSTORM_extras_lineDiameter(sectionFullFileNames, initParams, fitToMinBool, doubleLinesBool, linFitMethod, automaticSaveBool, automaticProcessBool)

% FUNCTION
%  Tool for determining line diameter.
% Cuts out a rotated rectangular region from the exported box section. Then
% convolves the localizations (Dirac deltas) with a Gaussian function and
% fits a curved line on this surface along along the ridge (the average
% height of the line is maximized) or along the gap (the same quantity
% minimized). Then creates a segmented line from the curved line and
% determines the distance of localizations of localizations from the
% segmented line. Finally creates a histogram plot from the acquired
% distances.

% % number of the exported sections
section_N=length(sectionFullFileNames);

lineDiam_histData=[];

%% Loop through the sections in APP
idxSection=1;
while idxSection<=section_N
    
    % name of the current histogram data file
    currentSectionDataFile=sectionFullFileNames{idxSection};
   
    [~,~,currentSectionDataFileExt]=fileparts(currentSectionDataFile);
    
    switch currentSectionDataFileExt
        case '.mat'
            sectionData=load(currentSectionDataFile,'-mat','export');
            sectionData.export.SectionRawDataFullFilename=currentSectionDataFile;
        otherwise
            error('Invalid file type.');
    end
    
    %% export->import
    SectionPosits = sectionData.export.SectionPosits;
    SectionParams = sectionData.export.SectionParams;
    SectionRawDataFullFilename = sectionData.export.SectionRawDataFullFilename;
    
    %% Reading the exported sesction from the workspace
    
    % localizations in nm:
    LOC_X=SectionPosits(:,1);
    LOC_Y=SectionPosits(:,2);
    
    ThompsonPrec=[SectionParams.x_std];
    PSFSize=[SectionParams.sig_x];
    % Filter Data
    LOC_X = LOC_X(PSFSize<=initParams.PSFmax);
    LOC_Y = LOC_Y(PSFSize<=initParams.PSFmax);
    ThompsonPrec = ThompsonPrec(PSFSize<=initParams.PSFmax);
    PSFSize = PSFSize(PSFSize<=initParams.PSFmax);
    
    %% Distance determination
    
    [lineDiam_lastParams, locLineDist, histCounts, histEdges, regionBool ]=fitCurveOnSurface(LOC_X, LOC_Y, PSFSize, initParams, fitToMinBool, doubleLinesBool, linFitMethod, automaticProcessBool);
    initParams=lineDiam_lastParams;
    
    % Filtering based on the PSF size
    %  BE AWARE: a the same filtering is also performed inside
    %  "manualLinearSetting" for LOC_X and LOC_Y
    ThompsonPrec = ThompsonPrec(PSFSize<=lineDiam_lastParams.PSFmax);
    PSFSize = PSFSize(PSFSize<=lineDiam_lastParams.PSFmax);
    
    %% Exporting the localization distances to an extendable workspace variable
    if ~isempty(lineDiam_histData)
        lineDiam_histData.locLineDist{end+1,1}=locLineDist';
        lineDiam_histData.ThompsonPrec{end+1,1}=ThompsonPrec(regionBool);
        lineDiam_histData.ThompsonPrec{end,1}(isnan(locLineDist))=NaN;
        lineDiam_histData.PSFSize{end+1,1}=PSFSize(regionBool);
        lineDiam_histData.PSFSize{end,1}(isnan(locLineDist))=NaN;
        lineDiam_histData.histCounts{end+1,1}=histCounts;
        lineDiam_histData.histEdges{end+1,1}=histEdges;
        lineDiam_histData.export_name{end+1,1} = sectionData.export.export_name;
        lineDiam_histData.SectionRawDataFullFilename{end+1,1} = sectionData.export.SectionRawDataFullFilename;
        lineDiam_histData.initLineFigure{end+1,1} = lineDiam_lastParams.fig10;
        lineDiam_histData.fittedCurveFigure{end+1,1} = lineDiam_lastParams.fig11;
        lineDiam_histData.distHistFigure{end+1,1} = lineDiam_lastParams.fig12;
        lineDiam_histData.lineDiam_lastParams{end+1,1} = lineDiam_lastParams;
    else
        lineDiam_histData.locLineDist=cell(1,1);
        lineDiam_histData.locLineDist{1,1}=locLineDist';
        lineDiam_histData.ThompsonPrec=cell(1,1);
        lineDiam_histData.ThompsonPrec{1,1}=ThompsonPrec(regionBool);
        lineDiam_histData.ThompsonPrec{1,1}(isnan(locLineDist))=NaN;
        lineDiam_histData.PSFSize=cell(1,1);
        lineDiam_histData.PSFSize{1,1}=PSFSize(regionBool);
        lineDiam_histData.PSFSize{1,1}(isnan(locLineDist))=NaN;
        lineDiam_histData.histCounts=cell(1,1);
        lineDiam_histData.histCounts{1,1}=histCounts;
        lineDiam_histData.histEdges=cell(1,1);
        lineDiam_histData.histEdges{1,1}=histEdges;
        lineDiam_histData.export_name{1,1} = sectionData.export.export_name;
        lineDiam_histData.SectionRawDataFullFilename{1,1} = sectionData.export.SectionRawDataFullFilename;
        lineDiam_histData.initLineFigure{1,1} = lineDiam_lastParams.fig10;
        lineDiam_histData.fittedCurveFigure{1,1} = lineDiam_lastParams.fig11;
        lineDiam_histData.distHistFigure{1,1} = lineDiam_lastParams.fig12;
        lineDiam_histData.lineDiam_lastParams{1,1} = lineDiam_lastParams;
    end
    
    disp('Line diameter data has been added.')
    
    % showing the figures:
    if ~automaticSaveBool
        figure(lineDiam_lastParams.fig10)
        figure(lineDiam_lastParams.fig11)
        figure(lineDiam_lastParams.fig12)
    end
	
    %% This part saves the raw and histogram data of localization distances from the fitted curve.
    
    % drawing the figure opening the dialog until the set parameters are not
    % accepted
    % question dialog in which one can accept the current angle,
    % set a new one manually or quit from the program
    
    if automaticSaveBool
        choice='Save All';
        [rawDataPath, rawDataName, ~]=fileparts(SectionRawDataFullFilename);
        mkdir(rawDataPath, 'histData');
        histDataFullFile=fullfile(rawDataPath, 'histData', filesep, strcat(rawDataName, '_lineDiameterHistData.mat'));
        
        
        % saving the figures
        % getting the path of the section data file
        [sectionDataPath, sectionDataName, ~]=fileparts(currentSectionDataFile);
        
        mkdir(sectionDataPath, 'initLine');
        figureName=fullfile(sectionDataPath, 'initLine', filesep, strcat(sectionDataName, '_initLine.jpg'));
        saveas(lineDiam_lastParams.fig10, figureName, 'jpg');
        
        mkdir(sectionDataPath, 'fittedCurve');
        figureName=fullfile(sectionDataPath, 'fittedCurve', filesep, strcat(sectionDataName, '_fittedCurve.jpg'));
        saveas(lineDiam_lastParams.fig11, figureName, 'jpg');
        
        mkdir(sectionDataPath, 'histFigure');
        figureName=fullfile(sectionDataPath, 'histFigure', filesep, strcat(sectionDataName, '_histFigure.jpg'));
        saveas(lineDiam_lastParams.fig12, figureName, 'jpg');
        
        
    else % select the saving path manually
        choice=choosedialog();
        if strcmp(choice,'Save All')
            if exist('SectionRawDataFullFilename', 'var')
                [rawDataPath, rawDataName, ~]=fileparts(SectionRawDataFullFilename);
                rawDataFullName=fullfile(rawDataPath, [rawDataName, '_lineDiameterHistData', '.mat']);
                [histDataName,histDataPath]=uiputfile('*.mat','Save Line Diameter Data As', rawDataFullName);
            else
                [histDataName,histDataPath]=uiputfile('*.mat','Save Line Diameter Data As');
            end
            
            histDataFullFile=fullfile(histDataPath, histDataName);
        end
    end
                
    
    switch choice
        case {'Discard Current', 'Reevaluate Current'}
            %if exist ('lineDiam_histData_workspace', 'var')
            if ~isempty(lineDiam_histData)
                if length(lineDiam_histData.locLineDist)==1
                    %clear('lineDiam_histData_workspace')
                    lineDiam_histData=[];
                else
                    lineDiam_histData.locLineDist(end)=[];
                    lineDiam_histData.ThompsonPrec(end)=[];
                    lineDiam_histData.PSFSize(end)=[];
                    lineDiam_histData.histCounts(end)=[];
                    lineDiam_histData.histEdges(end)=[];
                    lineDiam_histData.export_name(end) = sectionData.export.export_name;
                    lineDiam_histData.SectionRawDataFullFilename(end) = [];
                    lineDiam_histData.initLineFigure(end) = [];
                    lineDiam_histData.fittedCurveFigure(end) = [];
                    lineDiam_histData.distHistFigure(end) = [];
                    lineDiam_histData.lineDiam_lastParams(end) = [];
                end
            else
                Disp('Line diameter data does not exist in the workspace.')
            end
        case 'Save All'
            if ~isempty(lineDiam_histData)
                save(histDataFullFile,'lineDiam_histData', '-mat')
                disp('Line diameter data has been saved.')
                lineDiam_histData=[];
                
                
                % uncomment this for saving the figures when manually
                % saving the histogram data:
%                 if ~automaticSaveBool
%                     mkdir(histDataPath, 'initLine');
%                     figureName=fullfile(histDataPath, ['initLine', filesep, sectionDataName, '_initLine', '.jpg']);
%                     saveas(lineDiam_lastParams.fig10, figureName, 'jpg');
%                     
%                     mkdir(histDataPath, 'fittedCurve');
%                     figureName=fullfile(histDataPath, ['fittedCurve', filesep, sectionDataName, '_fittedCurve', '.jpg']);
%                     saveas(lineDiam_lastParams.fig11, figureName, 'jpg');
%                     
%                     mkdir(histDataPath, 'histFigure');
%                     figureName=fullfile(histDataPath, ['histFigure', filesep, sectionDataName, '_histFigure', '.jpg']);
%                     saveas(lineDiam_lastParams.fig12, figureName, 'jpg');
%                 end
                
                
            else
                Disp('Please export the data to workspace with rainSTORMs "ExportBoxSection".')
            end
            
        case 'Add Another or Leave'
            % do nothing
        case 'nothing'
            % do nothing
    end
    
    % closing the figures
    close(lineDiam_lastParams.fig10)
    close(lineDiam_lastParams.fig11)
    close(lineDiam_lastParams.fig12)
    
    if strcmp(choice, 'Reevaluate Current')
        idxSection=idxSection;
    else
       idxSection=idxSection+1; 
    end
    
end
    
    function choice=choosedialog(lineDiam_histData_workspace)
    
    d = dialog('Position',[300 300 615 100],'Name','Line diameter saving dialog');
    uicontrol('Parent',d,...
        'Style','text',...
        'Position',[10 40 210 40],...
        'HorizontalAlignment', 'left',...
        'String','Choosing what to do:');
    
    % button for deleting the data of the last section
    uicontrol('Parent',d,...
        'Position',[10 20 145 25],...
        'String','Discard Current',...
        'Callback',@(hObject, eventdata)deleteButton_callback(hObject, eventdata));
    
    % button for reevaluating the data of the last section
    uicontrol('Parent',d,...
        'Position',[160 20 145 25],...
        'String','Reevaluate Current',...
        'Callback',@(hObject, eventdata)reevaluateButton_callback(hObject, eventdata));
    
    % button for saving all the datas
    uicontrol('Parent',d,...
        'Position',[310 20 145 25],...
        'String','Save All',...
        'Callback',@(hObject, eventdata)saveButton_callback(hObject, eventdata));
    
    % button for closing the dialog
    uicontrol('Parent',d,...
        'Position',[460 20 145 25],...
        'String','Add Another or Leave',...
        'Callback',@(hObject,eventdata)quitButtonCallback(hObject,eventdata));
    
    choice='nothing';
    % Wait for d to close before running to completion
    uiwait(d);
    
        function deleteButton_callback(hObject, eventdata)
            choice='Discard Current';
            delete(gcf)
        end
    
        function reevaluateButton_callback(hObject, eventdata)
            choice='Reevaluate Current';
            delete(gcf)
        end
        
        function saveButton_callback(hObject, eventdata)
            choice='Save All';
            delete(gcf)
        end
    
        function quitButtonCallback(hObject, eventdata)
            choice='Add Another or Leave';
            delete(gcf)
        end
    
    end
    
end
